@@ -1,13 +1,13 @@
 --[[
 
-File l3build-aux.lua Copyright (C) 2018-2021 The LaTeX Project
+File l3build-aux.lua Copyright (C) 2018-2024 The LaTeX Project
 
 It may be distributed and/or modified under the conditions of the
 LaTeX Project Public License (LPPL), either version 1.3c of this
 license or (at your option) any later version.  The latest version
 of this license is in the file
 
-   http://www.latex-project.org/lppl.txt
+   https://www.latex-project.org/lppl.txt
 
 This file is part of the "l3build bundle" (The Work in LPPL)
 and all files in that bundle must be distributed together.
@@ -25,6 +25,7 @@ for those people who are interested.
 -- local safety guards and shortcuts
 
 local match = string.match
+local gsub  = string.gsub
 
 local pairs = pairs
 local print = print
@@ -32,6 +33,8 @@ local print = print
 local lookup = kpse.lookup
 
 local os_time = os.time
+local os_type = os.type
+
 --
 -- Auxiliary functions which are used by more than one main function
 --
@@ -133,9 +136,9 @@ function call(modules, target, opts)
   for _, module in ipairs(modules) do
     local text
     if module == "." and opts["config"] and #opts["config"]>0 then
-      text = " with configuration " .. opts["config"][1]
+      text = " and configuration \"" .. opts["config"][1] .. "\""
     else
-      text = " for module " .. module
+      text = " for module \"" .. module .. "\""
     end
     print("Running l3build with target \"" .. target .. "\"" .. text )
     local error_level = run(
@@ -179,3 +182,30 @@ function localtexmf()
   end
   return paths
 end
+
+-- Run a command after setting up the environmental variables
+function runcmd(cmd,dir,vars)
+  dir = dir or "."
+  dir = abspath(dir)
+  vars = vars or {}
+  -- Allow for local texmf files
+  local env = ""
+  if not match(checkformat,"^context$")  then
+    env = os_setenv .. " TEXMFCNF=." .. os_pathsep .. os_concat
+  end
+  local envpaths = "." .. localtexmf() .. os_pathsep
+    .. abspath(localdir) .. os_pathsep
+    .. dir .. (typesetsearch and os_pathsep or "")
+  -- Deal with spaces in paths
+  if os_type == "windows" and match(envpaths," ") then
+    envpaths = gsub(envpaths,'"','')
+  end
+  for _,var in pairs(vars) do
+    if env ~= "" then
+      env = env .. os_setenv .. " " .. var .. "=" .. envpaths .. os_concat
+    else
+      env = os_setenv .. " " .. var .. "=" .. envpaths
+    end
+  end
+  return run(dir,set_epoch_cmd(epoch, forcedocepoch) .. env .. cmd)
+end
